==> 2.1.0.tar.gz: lib/src/widgets/video_feed/video_feed_platform_view.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_feed_mode.dart';
import '../../models/video_feed_source.dart';
import '../../models/video_player_native_configuration.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_player_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'video_feed_platform_handler.dart';

typedef VideoFeedPlatformViewLoadFinishedCallback = void Function(
    FWError? error);
typedef VideoFeedPlatformViewCreatedCallback = void Function(
    VideoFeedPlatformHandler handler);

class VideoFeedPlatformView extends StatefulWidget {
  final VideoFeedSource source;
  final String? channel;
  final String? playlist;
  final String? playlistGroup;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final List<String>? productIds;
  final VideoFeedMode mode;
  final VideoFeedConfiguration? videoFeedConfiguration;
  final VideoPlayerNativeConfiguration? videoPlayerConfiguration;
  final AdConfiguration? adConfiguration;
  final VideoFeedPlatformViewLoadFinishedCallback?
      onVideoFeedPlatformViewLoadFinished;
  final VideoFeedPlatformViewCreatedCallback? onVideoFeedPlatformViewCreated;

  const VideoFeedPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.playlistGroup,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.mode = VideoFeedMode.row,
    this.videoFeedConfiguration,
    this.videoPlayerConfiguration,
    this.adConfiguration,
    this.onVideoFeedPlatformViewLoadFinished,
    this.onVideoFeedPlatformViewCreated,
  }) : super(key: key);

  @override
  State<VideoFeedPlatformView> createState() => _VideoFeedPlatformViewState();
}

class _VideoFeedPlatformViewState extends State<VideoFeedPlatformView>
    with AndroidPlatformViewMixin {
  final String _viewType = "fw.fondor.view/video_feed";
  Completer<VideoFeedPlatformHandler> _handlerCompleter = Completer();

  @override
  void initState() {
    super.initState();
  }

  @override
  void didUpdateWidget(covariant VideoFeedPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);
    // FWLoggerUtil.log("VideoFeedPlatformView didUpdateWidget");
    _handlerCompleter.future.then((handler) {
      handler.updateWidget(widget);
    });
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return buildAndroidPlatformView(_buildAndroidPlatformView);
    } else {
      return _buildIOSPlatformView(context);
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: key,
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _generateGestureRecognizers(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initSurfaceAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _generateGestureRecognizers(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.playlistGroup != null) {
      creationParams["playlistGroup"] = widget.playlistGroup!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (widget.productIds != null) {
      creationParams["productIds"] = widget.productIds!;
    }

    creationParams["mode"] = widget.mode.name;

    if (widget.videoFeedConfiguration != null) {
      creationParams["videoFeedConfiguration"] =
          VideoFeedConfigurationUtil.toJson(widget.videoFeedConfiguration!);
    }

    if (widget.videoPlayerConfiguration != null) {
      creationParams["videoPlayerConfiguration"] =
          VideoPlayerConfigurationUtil.toJson(widget.videoPlayerConfiguration!);
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<OneSequenceGestureRecognizer>> _generateGestureRecognizers() {
    return <Factory<OneSequenceGestureRecognizer>>{
      Factory<OneSequenceGestureRecognizer>(
        () => widget.mode == VideoFeedMode.row
            ? HorizontalDragGestureRecognizer()
            : VerticalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = VideoFeedPlatformHandler(viewId: viewId, widget: widget);
    if (_handlerCompleter.isCompleted) {
      _handlerCompleter = Completer();
    }
    _handlerCompleter.complete(handler);
    widget.onVideoFeedPlatformViewCreated?.call(handler);
  }
}

==> 2.4.6.tar.gz: lib/src/widgets/video_feed/video_feed_platform_view.dart <==
import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';
import '../../models/fw_native_error.dart';
import '../../models/ad_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/video_feed_configuration.dart';
import '../../models/video_feed_mode.dart';
import '../../models/video_feed_source.dart';
import '../../models/video_player_native_configuration.dart';
import '../../utils/model_utils/ad_configuration_util.dart';
import '../../utils/model_utils/video_feed_configuration_util.dart';
import '../../utils/model_utils/video_player_configuration_util.dart';
import '../mixins/android_platform_view_mixin.dart';
import 'video_feed_platform_handler.dart';
import 'video_feed_native_callbacks.dart';

typedef VideoFeedPlatformViewCreatedCallback = void Function(
    VideoFeedPlatformHandler handler);

class VideoFeedPlatformView extends StatefulWidget {
  final VideoFeedSource source;
  final String? channel;
  final String? playlist;
  final String? playlistGroup;
  final Map<String, List<String>>? dynamicContentParameters;
  final String? hashtagFilterExpression;
  final List<String>? productIds;
  final String? contentId;
  final VideoFeedMode mode;
  final VideoFeedConfiguration? videoFeedConfiguration;
  final VideoPlayerNativeConfiguration? videoPlayerConfiguration;
  final AdConfiguration? adConfiguration;
  final VideoFeedPlatformViewLoadFinishedCallback?
      onVideoFeedPlatformViewLoadFinished;
  final VideoFeedPlatformViewEmptyCallback? onVideoFeedPlatformViewEmpty;
  final VideoFeedPlatformViewCreatedCallback? onVideoFeedPlatformViewCreated;
  final VideoFeedPlatformViewDidStartPictureInPictureCallback?
      onVideoFeedPlatformViewDidStartPictureInPicture;
  final VideoFeedPlatformViewDidStopPictureInPictureCallback?
      onVideoFeedPlatformViewDidStopPictureInPicture;

  const VideoFeedPlatformView({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.playlistGroup,
    this.dynamicContentParameters,
    this.hashtagFilterExpression,
    this.productIds,
    this.contentId,
    this.mode = VideoFeedMode.row,
    this.videoFeedConfiguration,
    this.videoPlayerConfiguration,
    this.adConfiguration,
    this.onVideoFeedPlatformViewLoadFinished,
    this.onVideoFeedPlatformViewEmpty,
    this.onVideoFeedPlatformViewCreated,
    this.onVideoFeedPlatformViewDidStartPictureInPicture,
    this.onVideoFeedPlatformViewDidStopPictureInPicture,
  }) : super(key: key);

  @override
  State<VideoFeedPlatformView> createState() => _VideoFeedPlatformViewState();
}

class _VideoFeedPlatformViewState extends State<VideoFeedPlatformView>
    with AndroidPlatformViewMixin {
  final String _viewType = "fw.fondor.view/video_feed";
  VideoFeedPlatformHandler? _handler;
  @override
  void didUpdateWidget(covariant VideoFeedPlatformView oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (oldWidget.videoFeedConfiguration != widget.videoFeedConfiguration) {
      _handler?.updateVideoFeedConfiguration(widget.videoFeedConfiguration);
    }

    if (oldWidget.videoPlayerConfiguration != widget.videoPlayerConfiguration) {
      _handler?.updateVideoPlayerConfiguration(widget.videoPlayerConfiguration);
    }
  }

  @override
  void dispose() {
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    if (defaultTargetPlatform == TargetPlatform.android) {
      return buildAndroidPlatformView(_buildAndroidPlatformView);
    } else {
      return _buildIOSPlatformView(context);
    }
  }

  Widget _buildAndroidPlatformView(BuildContext context, ValueKey key) {
    final creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;
    const creationParamsCodec = StandardMessageCodec();
    return PlatformViewLink(
      key: key,
      viewType: _viewType,
      surfaceFactory:
          (BuildContext context, PlatformViewController controller) {
        return AndroidViewSurface(
          controller: controller as AndroidViewController,
          gestureRecognizers: _generateGestureRecognizers(),
          hitTestBehavior: PlatformViewHitTestBehavior.opaque,
        );
      },
      onCreatePlatformView: (PlatformViewCreationParams params) {
        return PlatformViewsService.initSurfaceAndroidView(
          id: params.id,
          viewType: _viewType,
          layoutDirection: layoutDirection,
          creationParams: creationParams,
          creationParamsCodec: creationParamsCodec,
          onFocus: () {
            params.onFocusChanged(true);
          },
        )
          ..addOnPlatformViewCreatedListener(params.onPlatformViewCreated)
          ..addOnPlatformViewCreatedListener(_onPlatformViewCreated)
          ..create();
      },
    );
  }

  Widget _buildIOSPlatformView(BuildContext context) {
    Map<String, dynamic> creationParams = _generateCreationParams();
    const layoutDirection = TextDirection.ltr;

    return UiKitView(
      viewType: _viewType,
      onPlatformViewCreated: _onPlatformViewCreated,
      layoutDirection: layoutDirection,
      creationParams: creationParams,
      creationParamsCodec: const StandardMessageCodec(),
      gestureRecognizers: _generateGestureRecognizers(),
    );
  }

  Map<String, dynamic> _generateCreationParams() {
    Map<String, dynamic> creationParams = <String, dynamic>{
      "source": widget.source.name,
    };

    if (widget.channel != null) {
      creationParams["channel"] = widget.channel!;
    }

    if (widget.playlist != null) {
      creationParams["playlist"] = widget.playlist!;
    }

    if (widget.playlistGroup != null) {
      creationParams["playlistGroup"] = widget.playlistGroup!;
    }

    if (widget.dynamicContentParameters != null) {
      creationParams["dynamicContentParameters"] =
          widget.dynamicContentParameters!;
    }

    if (widget.hashtagFilterExpression != null) {
      creationParams["hashtagFilterExpression"] =
          widget.hashtagFilterExpression!;
    }

    if (widget.productIds != null) {
      creationParams["productIds"] = widget.productIds!;
    }

    if (widget.contentId != null) {
      creationParams["contentId"] = widget.contentId!;
    }

    creationParams["mode"] = widget.mode.name;

    if (widget.videoFeedConfiguration != null) {
      creationParams["videoFeedConfiguration"] =
          VideoFeedConfigurationUtil.toJson(widget.videoFeedConfiguration!);
    }

    if (widget.videoPlayerConfiguration != null) {
      creationParams["videoPlayerConfiguration"] =
          VideoPlayerConfigurationUtil.toJson(widget.videoPlayerConfiguration!);
    }

    if (widget.adConfiguration != null) {
      creationParams["adConfiguration"] =
          AdConfigurationUtil.toJson(widget.adConfiguration!);
    }

    if (defaultTargetPlatform == TargetPlatform.android) {
      creationParams["marginRight"] = androidMarginRight;
    }

    return creationParams;
  }

  Set<Factory<OneSequenceGestureRecognizer>> _generateGestureRecognizers() {
    return <Factory<OneSequenceGestureRecognizer>>{
      Factory<OneSequenceGestureRecognizer>(
        () => widget.mode == VideoFeedMode.row
            ? HorizontalDragGestureRecognizer()
            : VerticalDragGestureRecognizer(),
      ),
    };
  }

  void _onPlatformViewCreated(int viewId) {
    final handler = VideoFeedPlatformHandler(
      viewId: viewId,
      callbacks: VideoFeedPlatformHandlerCallbacks(
        onPlatformViewLoadFinished: _onPlatformViewLoadFinished,
        onPlatformViewEmpty: _onPlatformViewEmpty,
        onPlatformViewDidStartPictureInPicture:
            _onPlatformViewDidStartPictureInPicture,
        onPlatformViewDidStopPictureInPicture:
            _onPlatformViewDidStopPictureInPicture,
      ),
    );
    _handler = handler;
    widget.onVideoFeedPlatformViewCreated?.call(handler);
  }

  void _onPlatformViewLoadFinished(FWNativeError? error) {
    widget.onVideoFeedPlatformViewLoadFinished?.call(error);
    if (error != null) {
      if (error.action == FWNativeErrorAction.loadingFirstPage.name) {
        widget.onVideoFeedPlatformViewEmpty?.call(error.toError());
      }
    }
  }

  void _onPlatformViewEmpty(FWError? error) {
    widget.onVideoFeedPlatformViewLoadFinished
        ?.call(FWNativeError.createNativeErrorWithError(error));
    widget.onVideoFeedPlatformViewEmpty?.call(error);
  }

  void _onPlatformViewDidStartPictureInPicture(FWError? error) {
    widget.onVideoFeedPlatformViewDidStartPictureInPicture?.call(error);
  }

  void _onPlatformViewDidStopPictureInPicture(FWError? error) {
    widget.onVideoFeedPlatformViewDidStopPictureInPicture?.call(error);
  }
}
